CON

  {Normal QTI Calibration}

  {Seesaw QTI Calibration}

  SpeedBumpsLeftThreshold = 1700
  SpeedBumpsMiddleThreshold = 1000
  SpeedBumpsRightThreshold = 1800

  {Bridge QTI Calibration}

  BridgeLeftThreshold = 1500
  BridgeMiddleThreshold = 1500
  BridgeRightThreshold = 1600

  {Sensitivity}
  
  Sensitivity = 30

  {Normal CPAL Calibration}

  LeftGreenRed = 36
  LeftGreenGreen = 71
  LeftGreenBlue = 74

  RightGreenRed = 37
  RightGreenGreen = 56
  RightGreenBlue = 76

  {Misc}

  FastSyncSpeed = 8
  SlowSyncSpeed = 18
  MediumSyncSpeed = 35

  UpdateDelayThreshold = 50

  UltrasonicMax = 100

OBJ

  QTI           : "QTIDriver"

  CPALLeft      : "ColorPAL"
  CPALRight     : "ColorPAL"
  
  Ping          : "Ping"

  Profiler[6]      : "Profiler" 
  
VAR

  byte LeftQTIPin, RightQTIPin, MiddleQTIPin, LeftColorPALPin, RightColorPALPin, CPALLeftPin, CPALRightPin
  
  byte LeftRed, LeftGreen, LeftBlue
  byte RightRed, RightGreen, RightBlue

  byte FastLoopCog, MediumLoopCog, SlowLoopCog

  long FastLoopSpeed, FastLoopSpeedFull, FastLoopWait
  long MediumLoopSpeed, MediumLoopSpeedFull, MediumLoopWait  
  long SlowLoopSpeed, SlowLoopSpeedFull, SlowLoopWait

  long stack[500]
  
  word LeftQTIRaw, MiddleQTIRaw, RightQTIRaw
  byte LeftQTIColor, MiddleQTIColor, RightQTIColor
  word LeftQTICalibration, MiddleQTICalibration, RightQTICalibration

  word leftRedCalibration, leftGreenCalibration, leftBlueCalibration
  word rightRedCalibration, rightGreenCalibration, rightBlueCalibration

  long MiddleCM, MiddleMM
  long RightCM, RightMM
  
  long SlowLoopCrashCounter
  long CPALLCntFromUpdate
  long CPALRCntFromUpdate

PUB Start(CPALPinLeft, QTILeftPin, QTIMiddlePin, QTIRightPin, CPALPinRight)

  CPALLeftPin := CPALPinLeft
  CPALRightPin := CPALPinRight
  LeftQTIPin := QTILeftPin
  MiddleQTIPin := QTIMiddlePin
  RightQTIPin := QTIRightPin
  LeftColorPALPin := CPALLeftPin
  RightColorPALPin := CPALRightPin

  'InitButtons
  LaunchCogs

'-----[ Cogs ]-----'

PUB LaunchCogs

  StartFastLoopCog
  StartMediumLoopCog
  StartSlowLoopCog

PUB StartFastLoopCog

  FastLoopCog := cognew(FastLoop, @stack[0])

PUB StopFastLoopCog

  cogstop(FastLoopCog)

PUB RestartFastLoopCog

  StopFastLoopCog
  waitcnt(clkfreq/100+cnt)
  StartFastLoopCog

PUB StartMediumLoopCog

  MediumLoopCog := cognew(MediumLoop, @stack[200])

PUB StopMediumLoopCog

  cogstop(MediumLoopCog)

PUB RestartMediumLoopCog

  StopMediumLoopCog
  waitcnt(clkfreq/100+cnt)
  StartMediumLoopCog

PUB StartSlowLoopCog

  SlowLoopCog := cognew(SlowLoop, @stack[100])

PUB StopSlowLoopCog

  cogstop(SlowLoopCog)

PUB RestartSlowLoopCog

  StopSlowLoopCog
  waitcnt(clkfreq/100+cnt)
  StartSlowLoopCog

PUB StopCogs

  StopFastLoopCog
  StopMediumLoopCog
  StopSlowLoopCog

'-----[ Misc] -----'

PUB InitButtons

  DIRA[16..18]~

PUB SetQTICalibration(l, m, r)

  LeftQTICalibration := l
  MiddleQTICalibration := m
  RightQTICalibration := r

PUB SetCPALCalibration(lred, lgreen, lblue, rred, rgreen, rblue)

  leftRedCalibration := lred
  leftGreenCalibration := lgreen
  leftBlueCalibration := lblue

  rightRedCalibration := rred
  rightGreenCalibration := rgreen
  rightBlueCalibration := rblue

PUB SetLeftCPALCalibration(red, green, blue)

  leftRedCalibration := red
  leftGreenCalibration := green
  leftBlueCalibration := blue

PUB SetRightCPALCalibration(red, green, blue)

  rightRedCalibration := red
  rightGreenCalibration := green
  rightBlueCalibration := blue

PUB FastLoop

  QTI.addQTI(LeftQTIPin)
  QTI.addQTI(MiddleQTIPin)  
  QTI.addQTI(RightQTIPin)

  repeat
    Profiler[1].StartTimer
    Profiler[0].StartTimer

    LeftQTIRaw := QTI.GetRaw(LeftQTIPin)
    MiddleQTIRaw := QTI.getRaw(MiddleQTIPin)
    RightQTIRaw := QTI.getRaw(RightQTIPin)

    Profiler[0].StopTimer
    FastLoopSpeed := Profiler.getTimeMs
    if FastLoopSpeed < FastSyncSpeed
      FastLoopWait := FastSyncSpeed - FastLoopSpeed
      pause(FastLoopWait)
    else
      FastLoopWait := -1
    Profiler[1].StopTimer
    FastLoopSpeedFull := Profiler[1].getTimeMs

PUB MediumLoop

  repeat
    Profiler[5].StartTimer
    Profiler[4].StartTimer

    MiddleMM := Ping.Millimeters(6)
    'RightMM := Ping.Millimeters(7)
    MiddleCM := MiddleMM/10
    'RightCM := RightMM/10
    MiddleMM <#= UltrasonicMax*10
    'RightMM <#= UltrasonicMax*10
    MiddleCM <#= UltrasonicMax
    'RightCM <#= UltrasonicMax

    SlowLoopCrashCounter++
    if SlowLoopCrashCounter > 30
      RestartSlowLoopCog
      SlowLoopCrashCounter := 0
      

    Profiler[4].StopTimer
    MediumLoopSpeed := Profiler[4].getTimeMs
    if MediumLoopSpeed < mediumSyncSpeed
      MediumLoopWait := mediumSyncSpeed - MediumLoopSpeed
      pause(MediumLoopWait)
    else
      MediumLoopWait := -1
    MediumLoopSpeedFull := Profiler[5].StopTimerX

PUB SlowLoop

  CPALLeft.Init(4)
  CPALRight.Init(5) 

  repeat
    Profiler[3].StartTimer
    Profiler[2].StartTimer

    '-----{Update Readings, and check for any crashes}-----'

    if LeftRed == CPALLeft.GetRed and LeftGreen == CPALLeft.GetGreen and LeftBlue == CPALLeft.GetBlue
      CPALLCntFromUpdate++
      if CPALLCntFromUpdate > 10
        CPALLeft.Restart(CPALLeftPin)
        CPALLCntFromUpdate := 0
    else
      LeftRed := CPALLeft.GetRed
      LeftGreen := CPALLeft.GetGreen
      LeftBlue := CPALLeft.GetBlue
      CPALLCntFromUpdate := 0

    if RightRed == CPALRight.GetRed and RightGreen == CPALRight.GetGreen and RightBlue == CPALRight.GetBlue
      CPALRCntFromUpdate++
      if CPALRCntFromUpdate > 10
        CPALRight.Restart(CPALRightPin)
        CPALRCntFromUpdate := 0
    else
      RightRed := CPALRight.GetRed
      RightGreen := CPALRight.GetGreen
      RightBlue := CPALRight.GetBlue
      CPALRCntFromUpdate := 0

    SlowLoopCrashCounter := 0

    Profiler[2].StopTimer
    SlowLoopSpeed := Profiler[2].getTimeMs
    if SlowLoopSpeed < SlowSyncSpeed
      SlowLoopWait := SlowSyncSpeed - SlowLoopSpeed
      pause(SlowLoopWait)
    else
      SlowLoopWait := -1
    Profiler[3].StopTimer
    SlowLoopSpeedFull := Profiler[3].getTimeMs

PUB GetLeftCrashCounter

  return cpalLCntFromUpdate

PUB GetRightCrashCounter

  return cpalRCntFromUpdate

PUB GetcpalCrashCounter

  return SlowLoopCrashCounter

PUB GetFastLoopSpeed

  return FastLoopSpeed

PUB GetMediumLoopSpeed

  return MediumLoopSpeed

PUB GetSlowLoopSpeed

  return SlowLoopSpeed

PUB GetFastLoopSpeedFull

  return FastLoopSpeedFull

PUB getMediumLastMsFull

  return MediumLoopSpeedFull

PUB GetSlowLoopSpeedFull

  return SlowLoopSpeedFull  

PUB GetFastWait

  return FastLoopWait

PUB GetMediumWait

  return MediumLoopWait

PUB GetSlowWait

  return SlowLoopWait

PUB IsFastLoopSlowerThanSync

  if FastLoopSpeed > FastSyncSpeed
    return true
  else
    return false

PUB IsSlowLoopSlowerThanSync

  if SlowLoopSpeed > SlowSyncSpeed
    return true
  else
    return false

PUB GetMiddleMM

  return MiddleMM

PUB GetRightMM

  return RightMM

PUB GetMiddleCM

  return MiddleCM

PUB GetRightCM

  return RightCM

PUB GetLeftCPALColor

  if LeftRed > leftRedCalibration - Sensitivity and LeftRed < leftRedCalibration + Sensitivity
    if LeftGreen > leftGreenCalibration - Sensitivity and LeftGreen < leftGreenCalibration + Sensitivity
      if LeftBlue > leftBlueCalibration - Sensitivity and LeftBlue < leftBlueCalibration + Sensitivity
        return 1
      else
        return 0
    else
      return 0
  else
    return 0

PUB GetRightCPALColor

  if RightRed > rightRedCalibration - Sensitivity and rightRedCalibration < rightGreenRed + Sensitivity
    if RightGreen > rightGreenCalibration - Sensitivity and rightGreenCalibration < RightGreenGreen + Sensitivity
      if RightBlue > rightBlueCalibration - Sensitivity and rightBlueCalibration < RightGreenBlue + Sensitivity
        return 1
      else
        return 0
    else
      return 0
  else
    return 0    

PUB GetLeftQTI

  return LeftQTIRaw

PUB GetMiddleQTI

  return MiddleQTIRaw

PUB GetRightQTI

  return RightQTIRaw

PUB GetLeftQTIColor

  if LeftQTIRaw > LeftQTICalibration
    return 1
  else
    return 0

PUB getMiddleQTIColor

  if MiddleQTIRaw > MiddleQTICalibration
    return 1
  else
    return 0

PUB getRightQTIColor

  if RightQTIRaw > RightQTICalibration
    return 1
  else
    return 0

PUB GetLeftRed

  return LeftRed

PUB GetLeftGreen

  return LeftGreen

PUB GetLeftBlue

  return LeftBlue

PUB GetRightRed

  return RightRed 

PUB GetRightGreen

  return RightGreen

PUB GetRightBlue

  return RightBlue

PUB GetArmButton

  return INA[16]

PUB GetButtonLeft

  return INA[17]

PUB GetButtonRight

  return INA[18]

PUB GetLeftCalibration

  return LeftQTICalibration

PUB GetMiddleCalibration

  return RightQTICalibration

PUB GetRightCalibration

  return RightQTICalibration

PUB GetLeftRedCPALCalibration

  return leftRedCalibration

PUB GetLeftGreenCPALCalibration

  return leftGreenCalibration

PUB GetLeftBlueCPALCalibration

  return leftBlueCalibration

PUB GetRightRedCPALCalibration

  return rightRedCalibration

PUB GetRightGreenCPALCalibration

  return rightGreenCalibration

PUB GetRightBlueCPALCalibration

  return rightBlueCalibration

PRI Pause(ms)

  waitcnt(clkfreq/(1000/ms)+cnt)
                                