CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  START_BIT = 1
  START_CONFIRMED = 2

  PING_ = 3
  PONG = 4

  PIN_DIR = 5
  PIN_VAL = 6

  PING_MIDDLE_MM = 7
  PING_RIGHT_MM = 8

  RAW_QTI_LEFT = 9
  RAW_QTI_MIDDLE = 10
  RAW_QTI_RIGHT = 11

  OPEN_CLAW = 12
  CLOSE_CLAW = 13

  ROBOT_FORWARD = 14
  ROBOT_BACKWARD = 15

  ROBOT_SPIN_LEFT = 16
  ROBOT_SPIN_RIGHT = 17

  MOTOR_LEFT_FORWARD = 18
  MOTOR_LEFT_BACKWARD = 19

  MOTOR_RIGHT_FORWARD = 20
  MOTOR_RIGHT_BACKWARD = 21

  ARM_UP = 22
  ARM_DOWN = 23

  BOTTOM_BUTTON = 24
  TOP_BUTTON = 25

  CPAL_LEFT_COLOR = 26
  CPAL_RIGHT_COLOR = 27

  CPAL_LEFT_RED = 28
  CPAL_LEFT_GREEN = 29
  CPAL_LEFT_BLUE = 30

  CPAL_RIGHT_RED = 31
  CPAL_RIGHT_GREEN = 32
  CPAL_RIGHT_BLUE = 33

  QTI_LEFT_COLOR = 34
  QTI_MIDDLE_COLOR = 35
  QTI_RIGHT_COLOR = 36

  UPDATE_TIME = 37

  WAIT = 38

  TOGGLE_PIN = 39

  REBOOT_ = 40

  ROBOT_HALT = 41

  LCD_CLEAR_SCREEN = 42
  LCD_FIRST_LINE = 43
  LCD_SECOND_LINE = 44
  LCD_THIRD_LINE = 45
  LCD_FORTH_LINE = 46
  LCD_CR = 47
  LCD_CLEAR_FIRST_LINE = 48
  LCD_CLEAR_SECOND_LINE = 49
  LCD_CLEAR_THIRD_LINE = 50
  LCD_CLEAR_FORTH_LINE = 51
  LCD_STR = 52
  LCD_DEC = 53

  PING_MIDDLE_CM = 54
  PING_RIGHT_CM = 55

  SET_LEFT_CPAL_CALIBRATION = 56
  SET_RIGHT_CPAL_CALIBRATION = 57

  SET_QTI_CALIBRATION = 58

OBJ

  Serial: "Serial"
  Sensor: "Sensors"
  Mtr: "HB"
  LCD: "LCD"
  
VAR

  long recv[100]
                   
PUB Main

  waitcnt(clkfreq*2+cnt)

  Serial.Start(25, 24, 0, 9600)
  LCD.Start(0)

  SendDec(START_BIT)
  RecvPacket(START_BIT)

  repeat
    LCD.clearScreen

    Serial.RxStr(@recv)
    LCD.Str(recv)   

  Sensor.Start(4, 9, 10, 11, 5)

  waitcnt(clkfreq*2+cnt)

  Loop

PUB Loop | cmd, temp1, temp2, temp3

  Serial.Start(25, 24, 0, 115200)
  LCD.Start(0)

  LCD.Str(String("Starting..."))
  LCD.CR

  SendDec(START_BIT)
  RecvPacket(START_BIT)

  LCD.Str(String("Connected!"))

  waitcnt(clkfreq+cnt)

  repeat
    LCD.ClearScreen
    Serial.RxStr(@recv)
    LCD.Str(recv)
    waitcnt(clkfreq/10+cnt)

  Mtr.Start(1, 2, 3)
    
  repeat
    cmd := RecvDec
    'LCD.Dec(Cmd)
    'LCD.Cr

    waitcnt(clkfreq/100+cnt)

    '-----{ Motors }-----'

    if cmd==ROBOT_HALT
      Mtr.Halt
    elseif cmd == ROBOT_FORWARD
      temp1 := RecvDec 'Speed
      Mtr.SetForward(temp1)
    elseif cmd == ROBOT_BACKWARD
      temp1 := RecvDec 'Speed
      Mtr.SetForward(temp1)
    elseif cmd == ROBOT_SPIN_LEFT
      temp1 := RecvDec 'Speed
      Mtr.SetSpinLeft(temp1)
    elseif cmd == ROBOT_SPIN_RIGHT
      temp1 := RecvDec 'Speed
      Mtr.SetSpinRight(temp1)

    if cmd == MOTOR_LEFT_FORWARD
      temp1 := RecvDec 'Speed
      Mtr.SetForwardLeft(temp1)
    elseif cmd == MOTOR_LEFT_BACKWARD
      temp1 := RecvDec 'Speed
      Mtr.SetForwardRight(temp1)
    elseif cmd == MOTOR_RIGHT_FORWARD
      temp1 := RecvDec 'Speed
      Mtr.SetBackwardLeft(temp1)
    elseif cmd == MOTOR_RIGHT_BACKWARD
      temp1 := RecvDec 'Speed
      Mtr.SetBackwardRight(temp1)

    '-----{ Ultrasonic }-----'

    if cmd == PING_MIDDLE_MM
      SendDec(Sensor.getMiddleMM)
    elseif cmd == PING_RIGHT_MM
      SendDec(Sensor.getRightMM)
    elseif cmd == PING_MIDDLE_CM
      SendDec(Sensor.getMiddleCM)
    elseif cmd == PING_RIGHT_CM
      SendDec(Sensor.getRightCM)

    '-----{ QTI }-----'

    if cmd == QTI_LEFT_COLOR
      SendDec(Sensor.getLeftQTIColor)
    elseif cmd == QTI_MIDDLE_COLOR
      SendDec(Sensor.getMiddleQTIColor)
    elseif cmd == QTI_RIGHT_COLOR
      SendDec(Sensor.getRightQTIColor)

    if cmd == RAW_QTI_LEFT
      SendDec(Sensor.getLeftQTI)
    elseif cmd == RAW_QTI_MIDDLE
      SendDec(Sensor.getMiddleQTI)
    elseif cmd == RAW_QTI_RIGHT
      SendDec(Sensor.getRightQTI)

    '-----{ ColorPAL }-----'

    if cmd == CPAL_LEFT_RED
      SendDec(Sensor.getLeftRed)
    elseif cmd == CPAL_LEFT_GREEN
      SendDec(Sensor.getLeftGreen)
    elseif cmd == CPAL_LEFT_BLUE
      SendDec(Sensor.getLeftBlue)

    if cmd == CPAL_RIGHT_RED
      SendDec(Sensor.getRightRed)
    elseif cmd == CPAL_RIGHT_GREEN
      SendDec(Sensor.getRightGreen)
    elseif cmd == CPAL_RIGHT_BLUE
      SendDec(Sensor.getRightBlue)

    if cmd == CPAL_LEFT_COLOR
      SendDec(Sensor.getLeftCPALColor)
    elseif cmd == CPAL_RIGHT_COLOR
      SendDec(Sensor.getRightCPALColor)   

    '-----{ LCD }-----
      
    if cmd == LCD_CLEAR_SCREEN
      LCD.ClearScreen
    elseif cmd == LCD_FIRST_LINE
      LCD.FirstLine
    elseif cmd == LCD_SECOND_LINE
      LCD.SecondLine
    elseif cmd == LCD_THIRD_LINE
      LCD.ThirdLine
    elseif cmd == LCD_FORTH_LINE
      LCD.ForthLine
    elseif cmd == LCD_CR
      LCD.CR
    elseif cmd == LCD_CLEAR_FIRST_LINE
      LCD.ClearLine(1)
    elseif cmd == LCD_CLEAR_SECOND_LINE
      LCD.ClearLine(2)
    elseif cmd == LCD_CLEAR_THIRD_LINE
      LCD.ClearLine(3)
    elseif cmd == LCD_CLEAR_FORTH_LINE
      LCD.ClearLine(4)
    elseif cmd == LCD_STR
      RecvStr
      LCD.Str(recv)
      LCD.Str(String("TEsty"))
    elseif cmd == LCD_DEC
      temp1 := RecvDec
      LCD.Dec(temp1)

    '-----{ CPAL Calibration }-----'
    if cmd == SET_LEFT_CPAL_CALIBRATION
      temp1 := RecvDec 'Red
      temp2 := RecvDec 'Green
      temp3 := RecvDec 'Blue
      Sensor.SetLeftCPALCalibration(temp1, temp2, temp3)
    elseif cmd == SET_RIGHT_CPAL_CALIBRATION
      temp1 := RecvDec 'Red
      temp2 := RecvDec 'Green
      temp3 := RecvDec 'Blue
      Sensor.SetRightCPALCalibration(temp1, temp2, temp3)
    elseif cmd == SET_QTI_CALIBRATION
      temp1 := RecvDec 'Left
      temp2 := RecvDec 'Middle
      temp3 := RecvDec 'Right
      Sensor.SetQTICalibration(temp1, temp2, temp3)

    '-----{ Misc }-----'
    if cmd==WAIT
      pause(RecvDec)
    elseif cmd == REBOOT_
      reboot
    
      
    
     

PRI SendDec(Dec)

  Serial.Dec(Dec)
  Serial.tx(13)

PRI SendStr(Str)

  Serial.Str(Str)
  Serial.Tx(13)

PRI RecvDec

  return Serial.RxDec

PRI RecvStr

  Serial.RxStr(@recv)

PRI RecvPacket(packet) | p

  repeat until p==packet
    p := RecvDec

PRI Pause(Ms)

  waitcnt((clkfreq/1000)*ms+cnt)